@@ -105,8 +105,8 @@ def cluster(
     )
 
     assert unit_index in gdf.columns, (
-        "The column given as unit_index:, "
-        f"{unit_index} was not found in the geodataframe. ",
+        "The column given as unit_index: "
+        f"({unit_index}) was not found in the geodataframe. ",
         "If you need only a single time period, you can pass " "`unit_index=gdf.index`",
     )
     if not cluster_kwargs:
@@ -196,6 +196,8 @@ def cluster(
             name=model_colname,
             temporal_index=temporal_index,
             unit_index=unit_index,
+            scaler=scaler,
+            pooling=pooling,
         )
         if return_model:
             return gdf, results
@@ -241,6 +243,8 @@ def cluster(
                 name=model_colname,
                 temporal_index=temporal_index,
                 unit_index=unit_index,
+                scaler=scaler,
+                pooling=pooling,
             )
             models[time] = results
         if return_model:
@@ -428,6 +432,8 @@ def regionalize(
             name=model_colname,
             temporal_index=temporal_index,
             unit_index=unit_index,
+            scaler=scaler,
+            pooling=None,
         )
         models[time] = results
 
