@@ -5,102 +5,54 @@ from .util import get_lehd
 from warnings import warn
 
 
-def get_ltdb(
-    datastore,
-    state_fips=None,
-    county_fips=None,
-    msa_fips=None,
-    fips=None,
-    boundary=None,
-    years="all",
-):
-    """Create a new Community from LTDB data.
-
-        Instiantiate a new Community from pre-harmonized LTDB data. To use
-        you must first download and register LTDB data with geosnap using
-        the `store_ltdb` function. Pass lists of states, counties, or any
-        arbitrary FIPS codes to create a community. All fips code arguments
-        are additive, so geosnap will include the largest unique set.
-        Alternatively, you may provide a boundary to use as a clipping
-        feature.
+def get_nces(datastore, years="1516", dataset="sabs"):
+    """Extract a subset of data from the National Center for Educational Statistics as a long-form geodataframe.
 
     Parameters
     ----------
-    state_fips : list or str
-        string or list of strings of two-digit fips codes defining states
-        to include in the study area.
-    county_fips : list or str
-        string or list of strings of five-digit fips codes defining
-        counties to include in the study area.
-    msa_fips : list or str
-        string or list of strings of fips codes defining
-        MSAs to include in the study area.
-    fips : list or str
-        string or list of strings of five-digit fips codes defining
-        counties to include in the study area.
-    boundary : geopandas.GeoDataFrame
-        geodataframe that defines the total extent of the study area.
-        This will be used to clip tracts lazily by selecting all
-        `GeoDataFrame.representative_point()`s that intersect the
-        boundary gdf
-    years : list of ints
-        list of years (decades) to include in the study data
-        (the default "all" is [1970, 1980, 1990, 2000, 2010]).
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
+    years : str, optional
+        set of academic years to return formatted as a 4-digit string representing the two years
+        from a single period of the academic calendar. For example, the 2015-2016 academic year
+        is represented as "1516". Defaults to "1516"
+    dataset : str, optional
+        which NCES dataset to query. Options include `sabs`, `districts`, or `schools`
+        Defaults to 'sabs'
 
     Returns
     -------
     geopandas.GeoDataFrame
-        long-form geodataframe (with repeated geometries for each time period)
-
+        long-form geodataframe with 'year' column representing each time period
 
     """
-    if years == "all":
-        years = [1970, 1980, 1990, 2000, 2010]
-    if isinstance(boundary, gpd.GeoDataFrame):
-        tracts = datastore.tracts_2010()[["geoid", "geometry"]]
-        ltdb = datastore.ltdb().reset_index()
-        if boundary.crs != tracts.crs:
-            warn(
-                "Unable to determine whether boundary CRS is WGS84 "
-                "if this produces unexpected results, try reprojecting"
-            )
-        tracts = tracts[tracts.representative_point().intersects(boundary.unary_union)]
-        gdf = ltdb[ltdb["geoid"].isin(tracts["geoid"])]
-        gdf = gpd.GeoDataFrame(gdf.merge(tracts, on="geoid", how="left"), crs=4326)
+    if isinstance(years, (str,)):
+        years = [int(years)]
+    elif isinstance(years, (int,)):
+        years = [years]
 
-    else:
-        gdf = _from_db(
-            data=datastore.ltdb(),
-            state_fips=state_fips,
-            county_fips=county_fips,
-            msa_fips=msa_fips,
-            fips=fips,
-            years=years,
-        )
-    return gdf.reset_index()
+    dflist = []
+    for year in years:
+        df = datastore.nces(year=year, dataset=dataset)
+        dflist.append(df)
+    gdf = pd.concat(dflist)
+    return gdf.reset_index(drop=True)
 
 
-def get_ncdb(
+def get_ejscreen(
     datastore,
     state_fips=None,
     county_fips=None,
     msa_fips=None,
     fips=None,
-    boundary=None,
     years="all",
 ):
-    """Create a new Community from NCDB data.
-
-        Instiantiate a new Community from pre-harmonized NCDB data. To use
-        you must first download and register LTDB data with geosnap using
-        the `store_ncdb` function. Pass lists of states, counties, or any
-        arbitrary FIPS codes to create a community. All fips code arguments
-        are additive, so geosnap will include the largest unique set.
-        Alternatively, you may provide a boundary to use as a clipping
-        feature.
+    """Extract a subset of data from the EPA EJSCREEN as a long-form geodataframe.
 
     Parameters
     ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
     state_fips : list or str
         string or list of strings of two-digit fips codes defining states
         to include in the study area.
@@ -111,73 +63,66 @@ def get_ncdb(
         string or list of strings of fips codes defining
         MSAs to include in the study area.
     fips : list or str
-        string or list of strings of five-digit fips codes defining
-        counties to include in the study area.
-    boundary : geopandas.GeoDataFrame
-        geodataframe that defines the total extent of the study area.
-        This will be used to clip tracts lazily by selecting all
-        `GeoDataFrame.representative_point()`s that intersect the
-        boundary gdf
-    years : list of ints
-        list of years (decades) to include in the study data
-        (the default is all available [1970, 1980, 1990, 2000, 2010]).
+        string or list of strings of fips codes (any length) defining
+        census units to include in the study area.
+    years : str, optional
+        list of years to include in the extract. Following Census convention, years
+        are named by the conclusion of the 5-year period. For example the 2011-2015
+        sample is represented as `2015`. Defaults to "all" which includes every dataset
+        available (curently 2012-2019)
 
     Returns
     -------
-    Community
-        Community with NCDB data
-
+    geopandas.GeoDataFrame
+        long-form geodataframe with 'year' column representing each time period
     """
     if years == "all":
-        years = [1970, 1980, 1990, 2000, 2010]
-    if isinstance(boundary, gpd.GeoDataFrame):
-        tracts = datastore.tracts_2010()[["geoid", "geometry"]]
-        ncdb = datastore.ncdb().reset_index()
-        if boundary.crs != tracts.crs:
-            warn(
-                "Unable to determine whether boundary CRS is WGS84 "
-                "if this produces unexpected results, try reprojecting"
-            )
-        tracts = tracts[tracts.representative_point().intersects(boundary.unary_union)]
-        gdf = ncdb[ncdb["geoid"].isin(tracts["geoid"])]
-        gdf = gpd.GeoDataFrame(gdf.merge(tracts, on="geoid", how="left"), crs=4326)
+        years = list(range(2015, 2021))
 
-    else:
-        gdf = _from_db(
-            data=datastore.ncdb(),
+    elif isinstance(years, (str,)):
+        years = [int(years)]
+    elif isinstance(years, (int,)):
+        years = [years]
+
+    msa_counties = _msa_to_county(datastore, msa_fips)
+
+    states, allfips = _fips_to_states(state_fips, county_fips, msa_counties, fips)
+
+    dflist = []
+    for year in years:
+        df = datastore.ejscreen(
+            states=states,
+            year=year,
+        )
+        df = _fips_filter(
             state_fips=state_fips,
             county_fips=county_fips,
             msa_fips=msa_fips,
             fips=fips,
-            years=years,
+            data=df,
         )
-
-    return gdf.reset_index()
+        dflist.append(df)
+    gdf = pd.concat(dflist)
+    return gdf.reset_index(drop=True)
 
 
-def get_census(
+def get_acs(
     datastore,
+    level="bg",
     state_fips=None,
     county_fips=None,
     msa_fips=None,
     fips=None,
-    boundary=None,
     years="all",
     constant_dollars=True,
-    currency_year=2015,
+    currency_year=2019,
 ):
-    """Create a new Community from original vintage US Census data.
-
-        Instiantiate a new Community from . To use
-        you must first download and register census data with geosnap using
-        the `store_census` function. Pass lists of states, counties, or any
-        arbitrary FIPS codes to create a community. All fips code arguments
-        are additive, so geosnap will include the largest unique set.
-        Alternatively, you may provide a boundary to use as a clipping
-        feature.
+    """Extract a subset of data from the American Community Survey (ACS).
 
     Parameters
     ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
     state_fips : list or str, optional
         string or list of strings of two-digit fips codes defining states
         to include in the study area.
@@ -205,10 +150,235 @@ def get_census(
         If adjusting for inflation, this parameter sets the year in which dollar values will
         be expressed
 
+    Returns
+    -------
+    geopandas.GeoDataFrame
+        long-form geodataframe with 'year' column representing each time period
+    """
+    inflate_cols = [
+        "median_home_value",
+        "median_contract_rent",
+        "per_capita_income",
+        "median_household_income",
+    ]
+
+    if years == "all":
+        years = list(range(2012, 2020))
+
+    elif isinstance(years, (str,)):
+        years = [int(years)]
+    elif isinstance(years, (int,)):
+        years = [years]
+
+    msa_counties = _msa_to_county(datastore, msa_fips)
+
+    states, allfips = _fips_to_states(state_fips, county_fips, msa_counties, fips)
+
+    dflist = []
+    for year in years:
+        df = datastore.acs(
+            level=level,
+            states=states,
+            year=year,
+        )
+        df = _fips_filter(
+            state_fips=state_fips,
+            county_fips=county_fips,
+            msa_fips=msa_fips,
+            fips=fips,
+            data=df,
+        )
+        if constant_dollars:
+            try:
+                df = adjust_inflation(df, inflate_cols, year, currency_year)
+            except:
+                warn(
+                    "Currency columns unavailable at this resolution; not adjusting for inflation"
+                )
+        dflist.append(df)
+    gdf = pd.concat(dflist)
+    return gdf.reset_index(drop=True)
+
+
+def get_ltdb(
+    datastore,
+    state_fips=None,
+    county_fips=None,
+    msa_fips=None,
+    fips=None,
+    boundary=None,
+    years="all",
+):
+    """Extract a subset of data from the Longitudinal Tract Database (LTDB) as a long-form geodataframe.
+
+    Parameters
+    ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
+    state_fips : list or str
+        string or list of strings of two-digit fips codes defining states
+        to include in the study area.
+    county_fips : list or str
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    msa_fips : list or str
+        string or list of strings of fips codes defining
+        MSAs to include in the study area.
+    fips : list or str
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    boundary : geopandas.GeoDataFrame
+        geodataframe that defines the total extent of the study area.
+        This will be used to clip tracts lazily by selecting all
+        `GeoDataFrame.representative_point()`s that intersect the
+        boundary gdf
+    years : list of ints
+        list of years (decades) to include in the study data
+        (the default "all" is [1970, 1980, 1990, 2000, 2010]).
+
+    Returns
+    -------
+    geopandas.GeoDataFrame
+        long-form geodataframe with 'year' column representing each time period
+    """
+    if years == "all":
+        years = [1970, 1980, 1990, 2000, 2010]
+    if isinstance(boundary, gpd.GeoDataFrame):
+        tracts = datastore.tracts_2010()[["geoid", "geometry"]]
+        ltdb = datastore.ltdb().reset_index()
+        if not boundary.crs.equals(4326):
+            boundary = boundary.copy().to_crs(4326)
+        tracts = tracts[tracts.representative_point().intersects(boundary.unary_union)]
+        gdf = ltdb[ltdb["geoid"].isin(tracts["geoid"])]
+        gdf = gpd.GeoDataFrame(gdf.merge(tracts, on="geoid", how="left"), crs=4326)
+        gdf = gdf[gdf['year'].isin(years)]
+
+    else:
+        gdf = _from_db(
+            data=datastore.ltdb(),
+            state_fips=state_fips,
+            county_fips=county_fips,
+            msa_fips=msa_fips,
+            fips=fips,
+            years=years,
+        )
+    return gdf.reset_index()
+
+
+def get_ncdb(
+    datastore,
+    state_fips=None,
+    county_fips=None,
+    msa_fips=None,
+    fips=None,
+    boundary=None,
+    years="all",
+):
+    """Extract a subset of data from the Neighborhood Change Database (NCDB).
+
+    Parameters
+    ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
+    state_fips : list or str
+        string or list of strings of two-digit fips codes defining states
+        to include in the study area.
+    county_fips : list or str
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    msa_fips : list or str
+        string or list of strings of fips codes defining
+        MSAs to include in the study area.
+    fips : list or str
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    boundary : geopandas.GeoDataFrame
+        geodataframe that defines the total extent of the study area.
+        This will be used to clip tracts lazily by selecting all
+        `GeoDataFrame.representative_point()`s that intersect the
+        boundary gdf
+    years : list of ints
+        list of years (decades) to include in the study data
+        (the default is all available [1970, 1980, 1990, 2000, 2010]).
+
     Returns
     -------
     Community
-        Community with unharmonized census data
+        long-form geodataframe with 'year' column representing each time period
+    """
+    if years == "all":
+        years = [1970, 1980, 1990, 2000, 2010]
+    if isinstance(boundary, gpd.GeoDataFrame):
+        tracts = datastore.tracts_2010()[["geoid", "geometry"]]
+        ncdb = datastore.ncdb().reset_index()
+        if not boundary.crs.equals(4326):
+            boundary = boundary.copy().to_crs(4326)
+        tracts = tracts[tracts.representative_point().intersects(boundary.unary_union)]
+        gdf = ncdb[ncdb["geoid"].isin(tracts["geoid"])]
+        gdf = gpd.GeoDataFrame(gdf.merge(tracts, on="geoid", how="left"), crs=4326)
+        gdf = gdf[gdf['year'].isin(years)]
+
+    else:
+        gdf = _from_db(
+            data=datastore.ncdb(),
+            state_fips=state_fips,
+            county_fips=county_fips,
+            msa_fips=msa_fips,
+            fips=fips,
+            years=years,
+        )
+
+    return gdf.reset_index()
+
+
+def get_census(
+    datastore,
+    state_fips=None,
+    county_fips=None,
+    msa_fips=None,
+    fips=None,
+    boundary=None,
+    years="all",
+    constant_dollars=True,
+    currency_year=2015,
+):
+    """Extract a subset of data from the decennial U.S. Census as a long-form geodataframe.
+
+    Parameters
+    ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
+    state_fips : list or str, optional
+        string or list of strings of two-digit fips codes defining states
+        to include in the study area.
+    county_fips : list or str, optional
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    msa_fips : list or str, optional
+        string or list of strings of fips codes defining
+        MSAs to include in the study area.
+    fips : list or str, optional
+        string or list of strings of five-digit fips codes defining
+        counties to include in the study area.
+    boundary : geopandas.GeoDataFrame, optional
+        geodataframe that defines the total extent of the study area.
+        This will be used to clip tracts lazily by selecting all
+        `GeoDataFrame.representative_point()`s that intersect the
+        boundary gdf
+    years : list of ints, required
+        list of years to include in the study data
+        (the default is [1990, 2000, 2010]).
+    constant_dollars : bool, optional
+        whether to standardize currency columns to constant dollars. If true,
+        each year will be expressed in dollars set by the `currency_year` parameter
+    currency_year : int, optional
+        If adjusting for inflation, this parameter sets the year in which dollar values will
+        be expressed
+
+    Returns
+    -------
+    geopandas.GeoDataFrame
+        long-form geodataframe with 'year' column representing each time period
 
     """
     if years == "all":
@@ -258,11 +428,8 @@ def get_census(
     tracts = pd.concat(tracts, sort=False)
 
     if isinstance(boundary, gpd.GeoDataFrame):
-        if boundary.crs != tracts.crs:
-            warn(
-                "Unable to determine whether boundary CRS is WGS84 "
-                "if this produces unexpected results, try reprojecting"
-            )
+        if not boundary.crs.equals(4326):
+            boundary = boundary.copy().to_crs(4326)
         tracts = tracts[tracts.representative_point().intersects(boundary.unary_union)]
         gdf = tracts.copy()
 
@@ -292,7 +459,7 @@ def get_census(
             newtracts.append(df)
         gdf = pd.concat(newtracts)
 
-    return gdf
+    return gdf.reset_index(drop=True)
 
 
 def get_lodes(
@@ -305,17 +472,13 @@ def get_lodes(
     years=2015,
     dataset="wac",
 ):
-    """Create a new Community from Census LEHD/LODES data.
+    """Extract a subset of data from Census LEHD/LODES .
 
-        Instantiate a new Community from LODES data.
-        Pass lists of states, counties, or any
-        arbitrary FIPS codes to create a community. All fips code arguments
-        are additive, so geosnap will include the largest unique set.
-        Alternatively, you may provide a boundary to use as a clipping
-        feature.
 
     Parameters
     ----------
+    datastore : geosnap.DataStore
+        an instantiated DataStore object
     state_fips : list or str, optional
         string or list of strings of two-digit fips codes defining states
         to include in the study area.
@@ -343,8 +506,8 @@ def get_lodes(
 
     Returns
     -------
-    Community
-        Community with LODES data
+    geopandas.GeoDataFrame
+        long-form geodataframe with 'year' column representing each time period
 
     """
     if isinstance(years, (str,)):
@@ -356,6 +519,9 @@ def get_lodes(
     msa_counties = _msa_to_county(datastore, msa_fips)
 
     states, allfips = _fips_to_states(state_fips, county_fips, msa_counties, fips)
+    if boundary:
+        if not boundary.crs.equals(4326):
+            boundary = boundary.copy().to_crs(4326)
 
     if any(year < 2010 for year in years):
         gdf00 = datastore.blocks_2000(states=states, fips=(tuple(allfips)))
@@ -451,8 +617,10 @@ def _fips_to_states(state_fips, county_fips, msa_counties, fips):
     stateset = []
     for i in [state_fips, county_fips, msa_counties, fips]:
         if i:
-            if isinstance(i, (str,)):
+            if isinstance(i, str):
                 i = [i]
+            elif isinstance(i, int):
+                i = [str(i)]
             for each in i:
                 allfips.append(each)
                 stateset.append(each[:2])
